;;;; web-unity-cgi.scm
;
; Copyright (c) 2007, Peter Bex
; All rights reserved.
;
; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
; conditions are met:
;
;   Redistributions of source code must retain the above copyright notice, this list of conditions and the following
;     disclaimer. 
;   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following
;     disclaimer in the documentation and/or other materials provided with the distribution. 
;   Neither the name of the author nor the names of its contributors may be used to endorse or promote
;     products derived from this software without specific prior written permission. 
;
; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
; OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
; AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR
; CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
; CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
; SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
; THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
; OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
; POSSIBILITY OF SUCH DAMAGE.

(use web-unity)

(define (wu:cgi-get-method-params method env)
  (let ((content-length (string->number
			 (or (alist-ref "CONTENT_LENGTH" env equal? #f)
			     (alist-ref "HTTP_CONTENT_LENGTH" env equal? "0")))))
    (case method
      ((get)  (parse-encoded-arguments (alist-ref "QUERY_STRING" env equal? "")))
      ((post) (parse-encoded-arguments (read-string content-length (current-input-port))))
      (else   '()))))

(define (wu:get-header header #!optional default)
  (alist-ref (string-append "HTTP_" (string-translate header #\- #\_)) (current-environment) string-ci=? default))

(define (wu:write-headers)
  (printf "Status: ~A ~A\r\n" (car (wu:response-code)) (cdr (wu:response-code)))
  (for-each (lambda (pair)
	    (printf "~A: ~A\r\n" (car pair) (cdr pair)))
	  (wu:response-headers))
  (printf "\r\n")
  (wu:headers-sent #t))

(define (wu:run-handler dispatcher)
  (let ((m (string->symbol (string-downcase (alist-ref "REQUEST_METHOD" (current-environment) equal? "")))))
    (parameterize ((wu:params (wu:cgi-get-method-params m (current-environment)))
		   (wu:request-method m)
		   (wu:query-params (parse-encoded-arguments (alist-ref "QUERY_STRING" (current-environment) equal? "")))
		   (wu:path-info (alist-ref "PATH_INFO" (current-environment) equal? ""))
		   (wu:script-name (alist-ref "SCRIPT_NAME" (current-environment) equal? ""))
		   (wu:script-filename (alist-ref "SCRIPT_FILENAME" (current-environment) equal? ""))
		   (wu:response-headers (list (cons "Content-Type" "text/html")))
		   (wu:response-code (cons 200 "OK")))
      (wu:invoke-dispatcher dispatcher))))
