;;;; web-unity-scgi.scm
;
; Copyright (c) 2007, Peter Bex
; All rights reserved.
;
; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
; conditions are met:
;
;   Redistributions of source code must retain the above copyright notice, this list of conditions and the following
;     disclaimer. 
;   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following
;     disclaimer in the documentation and/or other materials provided with the distribution. 
;   Neither the name of the author nor the names of its contributors may be used to endorse or promote
;     products derived from this software without specific prior written permission. 
;
; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
; OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
; AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR
; CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
; CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
; SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
; THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
; OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
; POSSIBILITY OF SUCH DAMAGE.

(use srfi-69 scgi web-unity)

;; Bypass the resource handling of SCGI for now
;; TODO: Supply patch to add default handler to scgi
(scgi:resources (make-hash-table (constantly #t) (constantly #t)))

(define wu:scgi-environment (make-parameter #f))

(define (wu:get-header header #!optional default)
  (hash-table-ref/default (wu:scgi-environment) (string-append "HTTP_" (string-upcase (string-translate header #\- #\_))) default))

(define wu:user-dispatcher (make-parameter #f))

(define (wu:write-headers)
  (scgi:write-response-header
   (alist-ref "content-type" (wu:response-headers) string-ci=? "text/html")
   #f (wu:response-headers) (car (wu:response-code)) (cdr (wu:response-code)))
  (wu:headers-sent #t))

(scgi:add-resource '("/")
  (lambda (env pars)
    (parameterize ((wu:params (hash-table->alist pars))
		   (wu:request-method (string->symbol (string-downcase (hash-table-ref/default env "REQUEST_METHOD" ""))))
		   (wu:query-params (parse-encoded-arguments (hash-table-ref/default env "QUERY_STRING" "")))
		   (wu:path-info (hash-table-ref/default env "PATH_INFO" ""))
		   (wu:script-name (hash-table-ref/default env "SCRIPT_NAME" ""))
		   (wu:script-filename (hash-table-ref/default env "SCRIPT_FILENAME" ""))
		   (wu:scgi-environment env)
		   (wu:response-headers (list (cons "Content-Type" "text/html")))
		   (wu:response-code (cons 200 "OK")))
      (wu:invoke-dispatcher (wu:user-dispatcher)))))

(define (wu:run-handler d)
  (parameterize ((wu:user-dispatcher d))
    ((scgi:make-server
      (tcp-listen
       9000
       4
       "127.0.0.1")) #t)))
